/***************************************************************************
                          moac.cpp  -  description
                             -------------------
    begin                : Wed Sep 17 2003
    copyright            : (C) 2003 by Mike Studer
    email                : ab6cv@arrl.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "moac.h"

//***************************************************************************
// Everything from k2cmdr.cpp
//***************************************************************************
#include <iostream>
#include <iomanip>
#include <sstream>
#include <string>
#include <stdlib.h>
#include <stdio.h>
#include <termios.h>
#include <string.h>
#include <unistd.h>
#include "commport.h"

using namespace std;


moac::moac(void *arg) : d_arg(arg)
{
   tune_step = 1;  // smallest tuning step
   fil = 1;     // Widest filter
   modenum = 3; // For CW as default mode hehe
}


void  moac::set_keypress(){
    struct termios new_settings;
    tcgetattr(0, &stored_settings); // fd = 0 STDIN
    new_settings = stored_settings;
    /* Disable canonical mode, and set buffer size to 1 byte */
    new_settings.c_lflag &= (~ICANON);
    new_settings.c_lflag &= (~ECHO); // No echo
    new_settings.c_cc[VTIME] = 0;
    new_settings.c_cc[VMIN] = 1;
    tcsetattr(0,TCSANOW, &new_settings);
    return;
}

void moac::reset_keypress(){
    tcsetattr(0, TCSANOW, &stored_settings);
    return;
}

void moac::cmd( char * fff, int fd)
{
  int slen = strlen(fff);
  write(fd, fff, slen);
}

void moac::bootup()
{
    // set rf output to a reasonable level 5 watts LOL
    strcpy(CMD, "PC050;");
    cmd( CMD, *fd );

    // tune up 10 Hz to force K2 to send more complete status
    strcpy(CMD, "UP1;");
    cmd( CMD, *fd );

    // Zero out the RIT/XIT offset
    strcpy(CMD, "RC;");
    cmd( CMD, *fd );

    // LCD bits
    strcpy(CMD, "DS;");
    cmd( CMD, *fd );

    // Filters
    strcpy(CMD, "FW00001;");
    cmd( CMD, *fd );

    // AGC
    strcpy(CMD, "GT;");
    cmd( CMD, *fd );

    // NB
    strcpy(CMD, "NB;");
    cmd( CMD, *fd );

    // CW buffer
    strcpy(CMD, "KS;");
    cmd( CMD, *fd );
}
   
void* moac::chk4cmd(){
  ch  = '\0';
  cmd_clear();

while(1==1)
{
 cin >> ch;
 if(ch != '\0')
  {
   ch_value = ch;
//   cout << "Debug = " << dec << ch_value << endl; // DEBUG

// Is this a multi digit sequence?
   if (p1 == 27 & p2 == 0)
   {
      p2 = ch_value;
      // check for 2 digit cmd sequences here
      cmd_parse();
      ch_value = 27;
   } else
      {
         if( p1 == 27 & p2 != 0)
            {
               p3 = ch_value;
               cmd_parse();
               ch_value = 999; // turns off 3 digit sequence
            }
      }


   switch(ch_value)
   {
      case 98:
         // 'b' Band Change DOWN in frequency
         cout << "Band DOWN" << endl;
         strcpy(CMD, "SW03;");
         cmd( CMD, *fd );
         break;
      case 66:
         // 'B' Band Change UP in frequency
         cout << "Band UP" << endl;
         strcpy(CMD, "SW01;");
         cmd( CMD, *fd );
         break;
      case 115:
         // 's' S Meter query   
         strcpy(CMD, "BG;");
         cmd( CMD, *fd );
         break;
      case 102:
         filters();
         break;
      case 117:
         // 'u' USB
         strcpy(CMD, "MD2;");
         cmd( CMD, *fd );
         break;
      case 108:
         // 'l' LSB
         strcpy(CMD, "MD1;");
         cmd( CMD, *fd );
         break;
      case 99:
         // 'c' CW
         strcpy(CMD, "MD3;");
         cmd( CMD, *fd );
         break;
      case 121:
         // 'y' RTTY
         strcpy(CMD, "MD6;");
         cmd( CMD, *fd );
         break;
      case 114:
         // 'r' Reverse
         if(MD == "3")
          {
            strcpy(CMD, "MD7;");
            cmd( CMD, *fd );
          }else
          if(MD == "6")
          {
            strcpy(CMD, "MD9;");
            cmd( CMD, *fd );
          }
          break;      
      case 27:
         // Multi digit sequence!
         if(p1 != 27)
            {
               p1 = ch_value;
            }
         break;

      }
  }         
 }
}

void moac::cmd_clear()
{
 p1 = p2 = p3 = 0;
}

void moac::cmd_parse()
{
// MUST have a 27 in p1 to even use this routine
   if(p3 == 0 )
   {
      // Check for specific 2 digit cmd sequences
      if( p2 == 120 )
         {
            cout << "Program exit" << endl;
            reset_keypress();
            exit(0);

         }
   }

// 3 digit command sequence tests

// Check for F keys
if( p2 == 79 & p3 != 0 )
   {
//      cout << "F Key = " << ch_value << endl;   // DEBUG

// Check for specific F keys
      if(p3 == 80 )
      {
         // F1 key
         vfo_dd();
      }
// Ok done with checks for F keys we are interested in so clear out p1, p2, p3      
      cmd_clear();
   }

// Check for Arrow key
if( p2 == 91 & p3 != 0 )
   {
//      cout << "Arrow key = " << ch_value << endl;  //DEBUG
// Some of the F keys use this sequence too.

switch (p3)
{
   case 68:
      tune_down();
      break;
   case 67:
      tune_up();
      break;
   case 65:
      s_up();
      break;
   case 66:
      s_down();
      break;
}
      cmd_clear();
   }

}

void* chk4cmd_wrapper(void* This)
{
 return static_cast<moac*>(This)->chk4cmd();
}

void* queryrig_wrapper(void* This)
{
 return static_cast<moac*>(This)->queryrig();
} 

//***************************************************************************
// Everything from k2stat.cpp
//***************************************************************************

void moac::cmdparse( string rawcmd )
{
int freq_update = 0;

  // The first two letters of a K2 response denote its identity
  // So they are extracted here
  string cmdword = rawcmd.substr(0, 2);
  //  cout << "Cmd = " << cmdword << endl;
  string cmddata = rawcmd.substr(2, 100);
  //cout << "Cmd data = " << cmddata << endl;

  if(cmdword ==  "FA"){
    FA = cmddata;
  }
  if(cmdword ==  "FB"){
    FB = cmddata;
  }
  if(cmdword ==  "FR"){
    FR = cmddata;
  }
  if(cmdword ==  "FT"){
    FT = cmddata;
    freq_update = 1;
  }
  if(cmdword ==  "IF"){
    IF = cmddata;
  }
  if(cmdword ==  "AN"){
    AN = cmddata;
  }
  if(cmdword ==  "BG"){
    BG = cmddata;
    s_meter();
  }
  if(cmdword ==  "FW"){
    string t = cmddata.substr(0, 4);
    FW = t;
    int p;
    istringstream iss4;
    iss4.str(FW);
    iss4 >> p;
    cout << p << " Hz filter" << endl;
  }
  if(cmdword ==  "GT"){
    GT = cmddata;
  }
  if(cmdword ==  "KS"){
    KS = cmddata;
  }
  if(cmdword ==  "KY"){
    KY = cmddata;
  }
  if(cmdword ==  "LK"){
    LK = cmddata;
  }
  if(cmdword ==  "MD"){
    MD = cmddata;
    int u;
    istringstream iss0;
    iss0.str(MD);
    iss0 >> u;
    
    switch(u)
      {
         case 1:
            // LSB
            cout << "LSB" << endl;
            break;
         case 2:
            // USB
            cout << "USB" << endl;
            break;
         case 3:
            // CW
            cout << "CW" << endl;
            break;
         case 6:
            // RTTY
            cout << "RTTY" << endl;
            break;
         case 7:
            // CW-REV
            cout << "CW Reverse" << endl;
            break;
         case 9:
            // RTTY-REV
            cout << "RTTY Reverse" << endl;
            break;
         }
  }
  if(cmdword ==  "NB"){
    NB = cmddata;
  }
  if(cmdword ==  "PA"){
    PA = cmddata;
  }
  if(cmdword ==  "PC"){
    PC = cmddata;
  }
  if(cmdword ==  "PS"){
    PS = cmddata;
  }
  if(cmdword ==  "RA"){
    RA = cmddata;
  }
  if(cmdword ==  "RT"){
    RT = cmddata;
  }
  if(cmdword ==  "SQ"){
    SQ = cmddata;
  }
  if(cmdword ==  "XT"){
    XT = cmddata;
  }
  if(freq_update == 1)
  {
     // Need this in case someone tunes the rig by hand.
     vfo_dd();
     freq_update = 0;
  }
}


 void* moac::queryrig()
{
 commport comm;
 while( 1 == 1 ){
    rxdata =  comm.read_K2( *fd );
    if( *rxdata != "none"){
    // DATA UPDATE RECEIVED
//      cout << "RX data = " << *rxdata << endl;       // DEBUG this is the ANNOYING one. But useful.

      temp = *rxdata;
      cmdparse( temp ); // Take cmd rcvd and save the data
    }
  }
}


void moac::vfo_dd()
{
   string akhz, bkhz;

   if((FA.length() != 0) && (FB.length() != 0)){
   akhz = FA.substr(3,7);
   bkhz = FB.substr(3,7);
   akhz.insert(5, ".");
   bkhz.insert(5, ".");
   
   istringstream iss1;
   iss1.str(akhz);
   iss1 >> akhz_d;

   istringstream iss2;
   iss2.str(bkhz);
   iss2 >> bkhz_d;
   
   if( FR == FT )
   {
      cout << setprecision(2) << fixed;   // Stupid computer tried to round off the freq
      if( FT == "0" )
      {
         cout << akhz_d << " Khz. VFO A Receive and transmit" << endl;
      }
      if( FT == "1" )
      {
         cout << bkhz_d << " Khz. VFO B Receive and transmit" << endl;
      }
   }else
      {
         if(FT == "0")
         {
            cout << akhz_d << " Khz. VFO A Transmit " << endl;
            cout << bkhz_d << " Khz. VFO B Receive " << endl;
         }
         if(FT == "1")
         {
            cout << akhz_d << " Khz. VFO A Receive " << endl;
            cout << bkhz_d << " Khz. VFO B Transmit " << endl;
         }
      }

   }
   cout << endl;
}

void moac::s_meter()
{
    if(BG.length() != 0)
    {
      // Here is another reason why C++ is evil. String to int conversion. 
      int sval;
      istringstream iss;
      iss.str(BG);
      iss >> sval;
      cout << "S meter = "<< sval << endl;
    }    
}

void moac::tune_up()
{
   switch(tune_step)
    {
      case 1:
         strcpy(CMD, "UP1;");
         break;
      case 2:
         strcpy(CMD, "UP2;");
         break;
      case 3:
         strcpy(CMD, "UP3;");
         break;
      case 4:
         strcpy(CMD, "UP4;");
         break;
    }

  cmd( CMD, *fd );
}

void moac::tune_down()
{
   switch(tune_step)
    {
      case 1:
         strcpy(CMD, "DN1;");
         break;
      case 2:
         strcpy(CMD, "DN2;");
         break;
      case 3:
         strcpy(CMD, "DN3;");
         break;
      case 4:
         strcpy(CMD, "DN4;");
         break;
    }

  cmd( CMD, *fd );
}

void moac::s_up()
{
  tune_step++;
  if(tune_step >= 4)
  {
     tune_step = 4;
  }
}

void moac::s_down()
{
   tune_step--;
   if(tune_step <= 1)
   {
      tune_step = 1;
   }
}

void moac::filters()
{
   fil++;
   if(fil > 4)
   {
      fil = 1;
   }
   switch(fil)
   {
      case 1:
         strcpy(CMD, "FW00001;");
         break;
      case 2:
         strcpy(CMD, "FW00002;");
         break;
      case 3:
         strcpy(CMD, "FW00003;");
         break;
      case 4:
         strcpy(CMD, "FW00004;");
         break;
   }
  cmd( CMD, *fd );      
}




